import { Declaration, PluginCreator } from 'postcss';

type PrefixFunction = (decl: Declaration, target: 'prop' | 'value') => string;
interface UserDefinedOptions {
    prefix: string | PrefixFunction;
    /**
     * @description default equal prefix
     */
    propPrefix?: string | PrefixFunction;
    transformProp?: boolean;
    transformValue?: boolean;
    /**
     * @description ignore the prop (css node left side)
     */
    ignoreProp?: (decl: Declaration) => boolean | undefined;
    /**
     * @description ignore the value (css node right side)
     */
    ignoreValue?: (decl: Declaration) => boolean | undefined;
    /**
     * @description ignore the whole Declaration
     */
    ignoreDecl?: (decl: Declaration) => boolean | undefined;
    /**
     * @description ignore the customProperty in css value
     */
    ignoreValueCustomProperty?: (customProperty: string, decl: Declaration) => boolean | undefined;
}

declare function matchCustomPropertyFromValue(str: string, cb: (arr: RegExpExecArray, index: number) => void): void;
declare function makeCustomProperty(customProperty: string, prefix: string): string;
declare const postcssPlugin = "postcss-custom-property-prefixer";
declare function makePrefixFunction(prefix: UserDefinedOptions['prefix']): PrefixFunction;
declare const postcssCustomPropertyPrefixer: PluginCreator<UserDefinedOptions>;

export { postcssCustomPropertyPrefixer as default, makeCustomProperty, makePrefixFunction, matchCustomPropertyFromValue, postcssPlugin };
