function isObject(value) {
  return value !== null && typeof value === "object";
}
function _defu(baseObject, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObject, {}, namespace, merger);
  }
  const object = Object.assign({}, defaults);
  for (const key in baseObject) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const value = baseObject[key];
    if (value === null || value === void 0) {
      continue;
    }
    if (merger && merger(object, key, value, namespace)) {
      continue;
    }
    if (Array.isArray(value) && Array.isArray(object[key])) {
      object[key] = [...value, ...object[key]];
    } else if (isObject(value) && isObject(object[key])) {
      object[key] = _defu(
        value,
        object[key],
        (namespace ? `${namespace}.` : "") + key.toString(),
        merger
      );
    } else {
      object[key] = value;
    }
  }
  return object;
}
function createDefu(merger) {
  return (...arguments_) => (
    // eslint-disable-next-line unicorn/no-array-reduce
    arguments_.reduce((p, c) => _defu(p, c, "", merger), {})
  );
}
const defu = createDefu();

const PropResolvedMarkSymbol = Symbol("PropResolved");
const ValueResolvedMarkSymbol = Symbol("ValueResolved");
function addMark(decl, mark) {
  Reflect.defineProperty(decl, mark, {
    enumerable: false,
    value: true,
    configurable: true,
    writable: true
  });
}
function getMark(decl, mark) {
  return Reflect.get(decl, mark);
}

function matchCustomPropertyFromValue(str, cb) {
  let arr;
  let index = 0;
  const regex = /var\(\s*(--\w[\w-]*)/g;
  while ((arr = regex.exec(str)) !== null) {
    cb(arr, index);
    index++;
  }
}
function makeCustomProperty(customProperty, prefix) {
  return `--${prefix}${customProperty.slice(2)}`;
}
const postcssPlugin = "postcss-custom-property-prefixer";
function makePrefixFunction(prefix) {
  return typeof prefix === "string" ? () => {
    return prefix;
  } : prefix;
}
const postcssCustomPropertyPrefixer = (options) => {
  const {
    ignoreProp,
    ignoreValue,
    prefix: _prefix,
    transformProp,
    transformValue,
    ignoreDecl,
    ignoreValueCustomProperty,
    propPrefix: _propPrefix
  } = defu(options, {
    ignoreProp: () => false,
    ignoreValue: () => false,
    ignoreValueCustomProperty: () => false,
    ignoreDecl: () => false,
    transformProp: true,
    transformValue: true
  });
  const prefix = makePrefixFunction(_prefix);
  const propPrefix = _propPrefix === void 0 ? prefix : makePrefixFunction(_propPrefix);
  return {
    postcssPlugin,
    prepare() {
      return {
        DeclarationExit(decl) {
          if (ignoreDecl(decl)) {
            return;
          }
          if (transformProp && decl.prop.startsWith("--") && !ignoreProp(decl) && !getMark(decl, PropResolvedMarkSymbol)) {
            decl.prop = makeCustomProperty(decl.prop, propPrefix(decl, "prop"));
            addMark(decl, PropResolvedMarkSymbol);
          }
          if (transformValue && !ignoreValue(decl) && !getMark(decl, ValueResolvedMarkSymbol)) {
            let value = decl.value;
            matchCustomPropertyFromValue(decl.value, (arr) => {
              const customProperty = arr[1];
              if (ignoreValueCustomProperty(customProperty, decl)) {
                return;
              }
              value = value.replaceAll(customProperty, makeCustomProperty(customProperty, prefix(decl, "value")));
            });
            decl.value = value;
            addMark(decl, ValueResolvedMarkSymbol);
          }
        }
      };
    }
  };
};
postcssCustomPropertyPrefixer.postcss = true;

export { postcssCustomPropertyPrefixer as default, makeCustomProperty, makePrefixFunction, matchCustomPropertyFromValue, postcssPlugin };
